@@ -39,7 +39,7 @@
 #include "log.h"
 #include "sock.h"
 #include "ud_socket.h"
-
+#include "event.h"
 /*
  * What is a rule?  It's polymorphic, pretty much.
  */
@@ -76,7 +76,7 @@ static void free_rule(struct rule *r);
 static void lock_rules(void);
 static void unlock_rules(void);
 static sigset_t *signals_handled(void);
-static struct rule *parse_file(const char *file);
+static struct rule *parse_file(int fd_rule, const char *file);
 static struct rule *parse_client(int client);
 static int do_cmd_rule(struct rule *r, const char *event);
 static int do_client_rule(struct rule *r, const char *event);
@@ -92,7 +92,6 @@ acpid_read_conf(const char *confdir)
 {
 	DIR *dir;
 	struct dirent *dirent;
-	char *file = NULL;
 	int nrules = 0;
 	regex_t preg;
 	int rc = 0;
@@ -117,11 +116,10 @@ acpid_read_conf(const char *confdir)
 
 	/* scan all the files */
 	while ((dirent = readdir(dir))) {
-		int len;
 		struct rule *r;
 		struct stat stat_buf;
-
-		len = strlen(dirent->d_name);
+        char *file = NULL;  /* rename: filename */
+        int fd_rule;
 
 		/* skip "." and ".." */
 		if (strncmp(dirent->d_name, ".", sizeof(dirent->d_name)) == 0)
@@ -129,50 +127,60 @@ acpid_read_conf(const char *confdir)
 		if (strncmp(dirent->d_name, "..", sizeof(dirent->d_name)) == 0)
 			continue;
 
+        if (asprintf(&file, "%s/%s", confdir, dirent->d_name) < 0) {
+            acpid_log(LOG_ERR, "asprintf: %s", strerror(errno));
+            closedir(dir);
+            unlock_rules();
+            return -1;
+        }
+
 		/* skip any files that don't match the run-parts convention */
 		if (regexec(&preg, dirent->d_name, 0, NULL, 0) != 0) {
-			acpid_log(LOG_INFO, "skipping conf file %s/%s", 
-				confdir, dirent->d_name);
+			acpid_log(LOG_INFO, "skipping conf file %s", file);
+            free(file);
 			continue;
 		}
 
-		/* Compute the length of the full path name adding one for */
-		/* the slash and one more for the NULL. */
-		len += strlen(confdir) + 2;
+        /* ??? Check for DT_UNKNOWN and do this, then "else if" on not DT_REG
+               and do the same as the !S_ISREG() branch below. */
+        if (dirent->d_type != DT_REG) { /* may be DT_UNKNOWN ...*/
+            /* allow only regular files and symlinks to files */
+            if (fstatat(dirfd(dir), dirent->d_name, &stat_buf, 0) != 0) {
+			    acpid_log(LOG_ERR, "fstatat(%s): %s", file, strerror(errno));
+			    free(file);
+			    continue; /* keep trying the rest of the files */
+		    }
+		    if (!S_ISREG(stat_buf.st_mode)) {
+                acpid_log(LOG_INFO, "skipping non-file %s", file);
+                free(file);
+                continue; /* skip non-regular files */
+            }
+        }
 
-		file = malloc(len);
-		if (!file) {
-			acpid_log(LOG_ERR, "malloc(): %s", strerror(errno));
-			unlock_rules();
-			return -1;
-		}
-		snprintf(file, len, "%s/%s", confdir, dirent->d_name);
-
-		/* allow only regular files and symlinks to files */
-		if (stat(file, &stat_buf) != 0) {
-			acpid_log(LOG_ERR, "stat(%s): %s", file,
-				strerror(errno));
-			free(file);
-			continue; /* keep trying the rest of the files */
-		}
-		if (!S_ISREG(stat_buf.st_mode)) {
-			acpid_log(LOG_INFO, "skipping non-file %s", file);
-			free(file);
-			continue; /* skip non-regular files */
-		}
-
-		r = parse_file(file);
+        /* open the rule file (might want to move this into parse_file()?) */
+		if ((fd_rule = openat(dirfd(dir), dirent->d_name, 
+                              O_RDONLY|O_CLOEXEC|O_NONBLOCK)) == -1) {
+                /* something went _really_ wrong.. Not Gonna Happen(tm) */
+                acpid_log(LOG_ERR, "openat(%s): %s", file, strerror(errno));
+                free(file);
+                /* ??? Too extreme?  Why not just continue? */
+                closedir(dir);
+                unlock_rules();
+                return -1;
+        }
+        /* fd is closed by parse_file() */
+		r = parse_file(fd_rule, file);
 		if (r) {
 			enlist_rule(&cmd_list, r);
 			nrules++;
 		}
 		free(file);
 	}
+
 	closedir(dir);
 	unlock_rules();
 
-	acpid_log(LOG_INFO, "%d rule%s loaded",
-	    nrules, (nrules == 1)?"":"s");
+	acpid_log(LOG_INFO, "%d rule%s loaded", nrules, (nrules == 1)?"":"s");
 
 	return 0;
 }
@@ -219,7 +227,7 @@ acpid_cleanup_rules(int do_detach)
 }
 
 static struct rule *
-parse_file(const char *file)
+parse_file(int fd_rule, const char *file)
 {
 	FILE *fp;
 	char buf[512];
@@ -228,7 +236,8 @@ parse_file(const char *file)
 
 	acpid_log(LOG_DEBUG, "parsing conf file %s", file);
 
-	fp = fopen(file, "r");
+    /* r - read-only, e - O_CLOEXEC */
+	fp = fdopen(fd_rule, "re");
 	if (!fp) {
 		acpid_log(LOG_ERR, "fopen(%s): %s", file, strerror(errno));
 		return NULL;
@@ -710,9 +719,9 @@ safe_write(int fd, const char *buf, int len)
 	int ntries = NTRIES;
 
 	do {
-		r = write(fd, buf+ttl, len-ttl);
+		r = TEMP_FAILURE_RETRY (write(fd, buf+ttl, len-ttl) );
 		if (r < 0) {
-			if (errno != EAGAIN && errno != EINTR) {
+			if (errno != EAGAIN) {
 				/* a legit error */
 				return r;
 			}
